import json
from channels.generic.websocket import WebsocketConsumer
from asgiref.sync import async_to_sync
from .models import MemberModel, MessageModel
import time

class ChatConsumer(WebsocketConsumer):
	def connect(self):
		channel_id : str = self.scope['path'].split('/')[3]

		self.room_group_name = 'chat' + channel_id

		user = self.scope["user"]
		if (user.is_anonymous or not user.is_authenticated):
			return

		if MemberModel.objects.filter(member_id=user.pk, channel_id=int(channel_id)).count() != 1:
			return

		if (self.channel_layer == None):
			return

		async_to_sync(self.channel_layer.group_add)(
			self.room_group_name,
			self.channel_name
		)

		self.accept()


	def receive(self, text_data=None, bytes_data=None):
		if text_data == None:
			return

		text_data_json = json.loads(text_data)
		message = text_data_json['message']

		channel_id : int = int(self.scope['path'].split('/')[3])
		user = self.scope["user"]
		if (user.is_anonymous or not user.is_authenticated):
			return

		if MemberModel.objects.filter(member_id=user.pk, channel_id=channel_id).count() != 1:
			return

		if (self.channel_layer == None):
			return

		print(message)

		message_time : int = int(time.time() * 1000)
		async_to_sync(self.channel_layer.group_send)(
			self.room_group_name,
			{
				'type':'chat_message',
				'author_id':user.pk,
				'content':message,
				'time':message_time,
			}
		)
		
		new_message = MessageModel()
		new_message.channel_id = channel_id
		new_message.author_id = user.pk
		new_message.content = message
		new_message.time = message_time
		new_message.save()


	def chat_message(self, event):

		channel_id : int = int(self.scope['path'].split('/')[3])
		user = self.scope["user"]
		if (user.is_anonymous or not user.is_authenticated):
			return

		if MemberModel.objects.filter(member_id=user.pk, channel_id=channel_id).count() != 1:
			return

		self.send(text_data=json.dumps({
			'type':'chat',
			'author_id':event['author_id'],
			'content':event['content'],
			'time': event['time'],
		}))
